@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# Copyright (c) 2000, 2001                    RIPE NCC
+# Copyright (c) 2000,2001,2002               RIPE NCC
 #
 # All Rights Reserved
 #
@@ -25,25 +25,70 @@
 # It does not "build" everything per se, but you can always just
 # put a "gmake" as the last line.:)
 
+##
+## Required configuration variables
+##
+
+# Note: Suggested system-specific variables are defined below
+
+# Root directory of the binary installation
+TOPDIR=/usr/local/whoisd
+
 # Where is the glib-config script
 GLIBCONF=/usr/local/bin/glib-config
 
 # MySQL include and lib directories
-MYSQLINC=/usr/local/include/mysql
-MYSQLLIB=/usr/local/lib/mysql
+MYSQLINC=/usr/local/mysql/include/mysql
+MYSQLLIB=/usr/local/mysql/lib/mysql
+MYSQLBIN=/usr/local/mysql/bin
 
 # c-client build directory
-CCLIENTDIR=/usr/local/build/imap/c-client
+CCLIENTINC=/usr/local/include
+CCLIENTLIB=/usr/local/lib
 
 # gcc (it's needed by the RPSL parser)
 # All warnings on 
 GCC="gcc -Wall"
 
-## Environment variables to pass to configure
-#CC=$GCC
-# Solaris Workshop C
+
+##
+## System-specific configurations are defined here
+##
+
+# Solaris 8, with additional software installed in /usr/local
+#GLIBCONF=/usr/local/bin/glib-config
+#MYSQLINC=/usr/local/mysql/include/mysql
+#MYSQLLIB=/usr/local/mysql/lib/mysql
+#MYSQLBIN=/usr/local/mysql/bin
+#CCLIENTINC=/usr/local/include
+#CCLIENTLIB=/usr/local/lib
+#GPGCMD=/usr/local/bin/gpg
+
+# Red Hat Linux 7.2
+#GLIBCONF=/usr/bin/glib-config
+#MYSQLINC=/usr/include/mysql
+#MYSQLLIB=/usr/lib
+#MYSQLBIN=/usr/bin
+#CCLIENTINC=/usr/include/imap
+#CCLIENTLIB=/usr/lib
+#GPGCMD=/usr/bin/gpg
+#LIBS="-L/usr/kerberos/lib -lgssapi_krb5 -lssl"
+#export LIBS
+
+# FreeBSD 4.3, with additional software installed in /usr/local
+#GLIBCONF=/usr/local/bin/glib-config
+#MYSQLINC=/usr/local/mysql/include/mysql
+#MYSQLLIB=/usr/local/mysql/lib/mysql
+#MYSQLBIN=/usr/local/mysql/bin
+#CCLIENTINC=/usr/local/include
+#CCLIENTLIB=/usr/local/lib
+#GPGCMD=/usr/local/bin/gpg
+#CFLAGS="-I/usr/local/include/bind -L/usr/local/lib"  
+#export CFLAGS
+
+
+# To use Solaris Workshop C, uncomment these lines
 #CC="ucbcc -xCC"
-# Don't forget to export all the variables you pass
 #export CC
 
 #Supplementary arguments to configure: taken from command line
@@ -53,21 +98,19 @@ CONFIGARGS="$@"
 ./configure \
 --with-mysqlinc=$MYSQLINC \
 --with-mysqllib=$MYSQLLIB \
---with-cclientdir=$CCLIENTDIR \
+--with-mysqlbin=$MYSQLBIN \
+--with-cclientinc=$CCLIENTINC \
+--with-cclientlib=$CCLIENTLIB \
 --with-glibconfig=$GLIBCONF \
+--with-gpgcmd=$GPGCMD \
+--prefix=$TOPDIR \
 $CONFIGARGS
 
-CONF_EXIT=$?
-if [ $CONF_EXIT -ne 0 ]; then
-  exit $CONF_EXIT
+
+CONF_RET=$?
+if [ $CONF_RET -ne 0 ]
+then
+ echo "Configure failed. Exiting\n" 
+ exit $CONF_RET
 fi
 
-# Invoke the configure script for the RPSL parser:
-# it needs GCC.
-
-CC=$GCC
-export CC
-
-cd modules/up/src
-./configure $CONFIGARGS
-
