@@ -14,7 +14,7 @@ Modification History:
 ******************/
 
 /************************************
-Copyright (c) 2000                              RIPE NCC
+Copyright (c) 2000,2001,2002                         RIPE NCC
 
 All Rights Reserved
 
@@ -34,11 +34,10 @@ AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 ***************************************/
 
+#include "rip.h"
+
 #include <stdio.h>
 #include <string.h>
-#include "ca_defs.h"
-#include "ca_configFns.h"
-#include "ca_dictionary.h"
 
 /* #define DEBUG */
 
