@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 2.27 $
+  $Revision: 2.29 $
 
   mm - MIME Parser module. Functions to parse a mail message part,
   find if it is MIME-encapsulated, dispatch the part to the
@@ -11,7 +11,7 @@
   Design and implementation by: daniele@ripe.net
 
   ******************/ /******************
-  Copyright (c) 2000,2001                            RIPE NCC
+  Copyright (c) 2000,2001,2002                       RIPE NCC
  
   All Rights Reserved
   
@@ -921,7 +921,8 @@ Mail_Header_Field *get_mail_hdr_field (MAILSTREAM *stream,
 
   char *tmphdr;
   char tmpline[MAXBUFSIZE];
-  int i, j, c, tmpsize, titlesize;
+  int i, j, k, c, tmpsize, titlesize;
+  int continuation = 0;
   Mail_Header_Field *hdr_field;
   Mail_Header_Field *mhfp;
   Mail_Header_Field *newmhfp;
@@ -932,9 +933,10 @@ Mail_Header_Field *get_mail_hdr_field (MAILSTREAM *stream,
 
   tmpsize = strlen(tmphdr);
 
-  /* Length of the header title plus ": "*/
-  titlesize = strlen(hdr_title) + 2;
+  /* Length of the header title plus ":" */
+  titlesize = strlen(hdr_title) + 1;
 
+  continuation = 0;   /* to detect continuation lines */
   j = 0;
   /* also initialize tmpline */
   for(i = 0; i < MAXBUFSIZE; i++){
@@ -947,44 +949,49 @@ Mail_Header_Field *get_mail_hdr_field (MAILSTREAM *stream,
     {
       c = tmphdr[i];
       if (c == 10) /* EOL */
-	{
-	  if ((j > 1) || ((mhfp == NULL) && (i == tmpsize - 1))) /* j>1 and not j>0 because "\r" is always read;
-								  * The second option is needed for
-								  * the empty headers */
+	  {
+	    if ((j > 1) || ((mhfp == NULL) && (i == tmpsize - 1))) /* j>1 and not j>0 because "\r" is always read;
+								    * The second option is needed for
+								    * the empty headers */
 	    {
-	      newmhfp = (Mail_Header_Field *)UT_malloc(sizeof(Mail_Header_Field));
-	      newmhfp->next = NULL;
-	      newmhfp->field = (char *)UT_malloc(j + 2);
-	      if (j > 1)
-		/* We do not copy here the header title */
-		sprintf (newmhfp->field,"%s\n",tmpline + titlesize);
-	      else
-		sprintf (newmhfp->field,"\n");
+	        newmhfp = (Mail_Header_Field *)UT_malloc(sizeof(Mail_Header_Field));
+	        newmhfp->next = NULL;
+	        newmhfp->field = (char *)UT_malloc(j + 2);
+	        if (j > 1)
+            {
+              if ( ! continuation )
+		        /* We do not copy the header title from the first line */
+		        sprintf (newmhfp->field,"%s\n",tmpline + titlesize);
+              else
+		        /* There is no header title on continuation lines */
+		        sprintf (newmhfp->field,"%s\n",tmpline);
+            }
+	        else
+		      sprintf (newmhfp->field,"\n");
 
-
-	      if (mhfp == NULL)
-		{
-		  mhfp = newmhfp;
-		  hdr_field = newmhfp;
-		}
-	      else
-		{
-		  mhfp->next = newmhfp;
-		  mhfp = newmhfp;
-		}
+	        if (mhfp == NULL)
+		    {
+		      mhfp = newmhfp;
+		      hdr_field = newmhfp;
+		    }
+	        else
+		    {
+		      mhfp->next = newmhfp;
+		      mhfp = newmhfp;
+		    }
 	    }
-	  j = 0;	  
-    /* re-initialize tmpline */
-    for(i = 0; i < MAXBUFSIZE; i++){
-      tmpline[i] = '\0';
-    }
-
-	}
+        continuation = 1;   /* next time we are reading continuation lines */
+	    j = 0;	  
+        /* re-initialize tmpline */
+        for (k = 0; k < MAXBUFSIZE; k++)
+        {
+          tmpline[k] = '\0';
+        }
+	  }
       else
-	{
-	  sprintf (tmpline + j++,"%c", c);
-	}
-
+	  {
+	    sprintf (tmpline + j++,"%c", c);
+	  }
     }
 
   UT_free(tmphdr);
