@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.12 $
+  $Revision: 1.15 $
 
   NT (Notifications) module
 
@@ -11,7 +11,7 @@
   Modification History:
         engin (06/07/2000) Created.
   ******************/ /******************
-  Copyright (c) 2000                              RIPE NCC
+  Copyright (c) 2000,2001,2002                    RIPE NCC
  
   All Rights Reserved
   
@@ -31,10 +31,12 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  ***************************************/
 
+#ifndef NOTIFICATION_H
+#define NOTIFICATION_H 
+
 
 #include <stdarg.h>
 #include <stdio.h>
-#include <string.h>
 #include <strings.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -43,9 +45,9 @@
 #include "UP_util.h"
 
 typedef struct {
-      GSList * less_spec; 
-      GSList * exact_match;
-      GSList * more_spec;
+      GList * less_spec; 
+      GList * exact_match;
+      GList * more_spec;
 } overlap_routes;
 
 
@@ -101,7 +103,9 @@ void NT_log_ntfy_list( GHashTable * filehash, char * log_file);
 
 void NT_delete_ntfy_list( GHashTable * filehash);
 
-GSList * NT_gather_ntfy_addresses( const char * old_object, const char * new_object);
+GList * NT_gather_ntfy_addresses( const char * old_object, const char * new_object);
+
+GList *NT_compare_lists(GList *old_irts, GList *new_irts, int option);
 
 void NT_write_all_ntfs(char * old_object, char * new_object, char * formatted_object,
                        const char * tempdir, GHashTable * ntfy_hash,  
@@ -114,3 +118,5 @@ void NT_write_all_frwds(char * old_object, char * new_object, const char * tempd
 
 void NT_forw_create_req(const char * object);
 
+
+#endif
