@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.16 $
+  $Revision: 1.18 $
 
   gpg.h - header for gpg.c .
 
@@ -10,7 +10,7 @@
   Authors             : filippo@ripe.net
   OSs Tested          : Solaris 7
   ******************/ /******************
-  Copyright (c) 2000                              RIPE NCC
+  Copyright (c) 2000,2001,2002                    RIPE NCC
  
   All Rights Reserved
   
@@ -105,6 +105,7 @@ enum {
    iKO_NO_OPENPGP_DATA,
    iKO_NO_IN_FILES,
    iKO_SECRET_KEY_PRESENT,
+   iKO_MULTIPLE_KEYS,
    iKO_GENERALFAILURE
 };
 
@@ -131,6 +132,7 @@ extern "C" {
 
 void PA_VerifySignature(struct VerifySignObject *vSO);
 void PA_ImportKey(struct ImportKeyObject *iKO);
+void PA_RemoveKey_withKeyID(struct ImportKeyObject *iKO);
 void PA_RemoveKey(struct ImportKeyObject *iKO);
 void PA_Decrypt(struct ReadCryptedObject *rDO);
 EPNodePtr PA_ParseMessage(EPNodePtr ptr);
