@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.48 $
+  $Revision: 1.56 $
 
   Query command module (qc).  This is what the whois query gets stored as in
   memory.
@@ -11,7 +11,7 @@
   Author              : ottrey@ripe.net
   Modifications by    : marek@ripe.net
   ******************/ /******************
-  Copyright (c) 1999                              RIPE NCC
+  Copyright (c) 1999,2000,2001,2002               RIPE NCC
  
   All Rights Reserved
   
@@ -30,35 +30,20 @@
   AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
   ***************************************/
+
+#define QC_IMPL
+#include "rip.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
 
-#define QC_IMPL
-
-#include "query_command.h"
-#include "defs.h"
-#include "constants.h"
-#include "which_keytypes.h"
-#include "memwrap.h"
-
-#include "ca_configFns.h"
-#include "ca_dictionary.h"
-#include "ca_macros.h"
-#include "ca_srcAttribs.h"
-
-#include "getopt.h"
 
 #define MAX_OPT_ARG_C 20
 
-/*+ String sizes +*/
-#define STR_S   63
-#define STR_M   255
-#define STR_L   1023
-#define STR_XL  4095
-#define STR_XXL 16383
-
+extern char *suboptarg;
+extern int getsubopt(char **optionp, char * const *tokens, char **valuep);
 
 /*++++++++++++++++++++++++++++++++++++++
   make a list of sources. expects list to hold source handles
@@ -72,25 +57,35 @@
 char *
 qc_sources_list_to_string(GList *list)
 {
-  char *result = NULL;
-  int oldlen = 0;
-  GList *qitem;
+    GString *tmp;
+    GList *qitem;
+    char *result;
 
-  for( qitem = g_list_first(list);
-       qitem != NULL;
-       qitem = g_list_next(qitem)) {
-    ca_dbSource_t *source_hdl = (ca_dbSource_t *) ( qitem->data );
-    char *srcname = ca_get_srcname( source_hdl );
-    
-    dieif( wr_realloc( (void **)& result, oldlen + strlen(srcname) + 2)
-	   != UT_OK);
-    if(oldlen > 0) {
-      strcat(result, ",");
+    /* use GString for the result, because it resizes magically */
+    tmp = g_string_sized_new(STR_L);
+
+    /* loop through the list, creating a string with the names of the
+       sources separated by commas */
+    qitem = g_list_first(list);
+    if (qitem != NULL) {
+        for (;;) {
+            ca_dbSource_t *source_hdl = (ca_dbSource_t *) ( qitem->data );
+            char *srcname = ca_get_srcname( source_hdl );
+            g_string_append(tmp, srcname);
+
+            qitem = g_list_next(qitem);
+            if (qitem == NULL) {
+                break;
+            }
+
+            g_string_append_c(tmp, ',');
+        }
     }
-    strcat(result, srcname);
-  }
-
-  return result;
+      
+    /* move to a new buffer for the return */
+    result = UT_strdup(tmp->str);
+    g_string_free(tmp, TRUE);
+    return result;
 }
 
 /* QC_environ_to_string() */
@@ -109,32 +104,36 @@ qc_sources_list_to_string(GList *list)
   +html+ </UL></DL>
 
   ++++++++++++++++++++++++++++++++++++++*/
-char *QC_environ_to_string(Query_environ qe) {
+char *
+QC_environ_to_string(Query_environ qe) 
+{
+  char *sources;
+  char passed_addr[IP_ADDRSTR_MAX];
+  GString *tmp;
   char *result;
-  char *str1;
-  char str2[IP_ADDRSTR_MAX];
-  char result_buf[STR_XL];
 
-  str1 = qc_sources_list_to_string(qe.sources_list);
-  
-  if( IP_addr_b2a( &(qe.pIP), str2, IP_ADDRSTR_MAX) != IP_OK ) { 
-    *str2 = '\0';
+  /* convert the sources and the passed address (if any) to printable strings */
+  sources = qc_sources_list_to_string(qe.sources_list);
+  if( IP_addr_b2a( &(qe.pIP), passed_addr, IP_ADDRSTR_MAX) != IP_OK ) { 
+    passed_addr[0] = '\0';
   }
   
-  sprintf(result_buf, "host=%s, keep_connection=%s, sources=%s, version=%s%s%s", qe.condat.ip, 
-          qe.k?"on":"off", 
-          str1, 
-          (qe.version == NULL) ? "?" : qe.version,
-          *str2 == '\0' ? "" : ", passedIP=",
-          *str2 == '\0' ? "" : str2
-          );
+  /* format the environment info */
+  tmp = g_string_sized_new(STR_L);
+  g_string_sprintf(tmp, 
+                   "host=%s, keep_connection=%s, sources=%s, version=%s%s%s", 
+                   qe.condat.ip, 
+                   qe.k ? "on" : "off", 
+                   sources, 
+                   (qe.version == NULL) ? "?" : qe.version,
+                   passed_addr[0] == '\0' ? "" : ", passedIP=",
+                   passed_addr);
   
-  wr_free(str1);
+  /* move result to return buffer, and free up memory */
+  result = UT_strdup(tmp->str);
+  g_string_free(tmp, TRUE);
+  UT_free(sources);
 
-  dieif( wr_malloc((void **)&result, strlen(result_buf)+1) != UT_OK);  
-
-  strcpy(result, result_buf);
-  
   return result;
   
 } /* QC_environ_to_string() */
@@ -156,7 +155,6 @@ char *QC_environ_to_string(Query_environ qe) {
 
   ++++++++++++++++++++++++++++++++++++++*/
 char *QC_query_command_to_string(Query_command *query_command) {
-  char *result;
   char result_buf[STR_XL];
   char *str1;
   char *str2;
@@ -166,10 +164,11 @@ char *QC_query_command_to_string(Query_command *query_command) {
   str2 = MA_to_string(query_command->object_type_bitmap, DF_get_class_names());
   str3 = WK_to_string(query_command->keytypes_bitmap);
   
-  sprintf(result_buf, "Query_command : inv_attrs=%s, recursive=%s, object_type=%s, (e=%d,g=%d,l=%d,m=%d,q=%d,t=%d,v=%d,x=%d,F=%d,K=%d,L=%d,M=%d,R=%d,S=%d), possible keytypes=%s, keys=[%s]",
+  sprintf(result_buf, "Query_command : inv_attrs=%s, recursive=%s, object_type=%s, (c=%s,e=%d,g=%d,l=%d,m=%d,q=%d,t=%d,v=%d,x=%d,F=%d,K=%d,L=%d,M=%d,R=%d,S=%d), possible keytypes=%s, keys=[%s]",
           str1,
 	  query_command->recursive?"y":"n",
           str2,
+          query_command->c_irt_search ? "TRUE" : "FALSE",
           query_command->e,
           query_command->g,
           query_command->l,
@@ -186,14 +185,11 @@ char *QC_query_command_to_string(Query_command *query_command) {
           query_command->S,
           str3,
           query_command->keys);
-  wr_free(str1);
-  wr_free(str2);
-  wr_free(str3);
+  UT_free(str1);
+  UT_free(str2);
+  UT_free(str3);
 
-  dieif( wr_malloc((void **)&result, strlen(result_buf)+1) != UT_OK);  
-  strcpy(result, result_buf);
-
-  return result;
+  return UT_strdup(result_buf);
   
 } /* QC_query_command_to_string() */
 
@@ -224,7 +220,7 @@ static void log_command(char *query_str, Query_command *query_command) {
     str = QC_query_command_to_string(query_command);
     ER_dbg_va(FAC_QC, ASP_QC_BUILD,
 	      "query=[%s]   %s", query_str, str);
-    wr_free(str);
+    UT_free(str);
   }
 } /* log_command() */
 
@@ -247,14 +243,14 @@ static void log_command(char *query_str, Query_command *query_command) {
 void QC_environ_free(Query_environ *qe) {
   if (qe != NULL) {
     if (qe->version != NULL) {
-      wr_free(qe->version);
+      UT_free(qe->version);
     }
 
     if (qe->sources_list != NULL) {
       g_list_free(qe->sources_list); 
       qe->sources_list=NULL;
     }
-    wr_free(qe);
+    UT_free(qe);
   }
 } /* QC_environ_free() */
 
@@ -282,9 +278,9 @@ void QC_environ_free(Query_environ *qe) {
 void QC_free(Query_command *qc) {
   if (qc != NULL) {
     if (qc->keys != NULL) {
-      wr_free(qc->keys);
+      UT_free(qc->keys);
     }
-    wr_free(qc);
+    UT_free(qc);
   }
 } /* QC_free() */
 
@@ -325,7 +321,7 @@ int QC_fill(char *query_str,
   int opt_argc;
   gchar **opt_argv;
   char *value;
-  char *tmp_query_str;
+/*  char *tmp_query_str;*/
   unsigned key_length;
   int i;
   int index;
@@ -333,6 +329,7 @@ int QC_fill(char *query_str,
   A_Type_t attr;
   getopt_state_t *gst = NULL;
 
+  query_command->c_irt_search = FALSE;
   query_command->d = 0;
   query_command->e = 0;
   query_command->g = 0;
@@ -363,19 +360,20 @@ int QC_fill(char *query_str,
   /* This is so Marek can't crash me :-) */
   /* Side Effect - query keys are subsequently cut short to STR_S size. */
 
-  dieif( wr_calloc((void **)&tmp_query_str, 1, STR_S+1) != UT_OK);  
-  strncpy(tmp_query_str, query_str, STR_S);
+/*  tmp_query_str = (char *)UT_calloc(1, STR_S+1);
+  strncpy(tmp_query_str, query_str, STR_S);*/
 
   /* Create the arguments. */
   /* This allows only a maximum of MAX_OPT_ARG_C words in the query. */
-  opt_argv = g_strsplit(tmp_query_str, " ", MAX_OPT_ARG_C);
+/*  opt_argv = g_strsplit(tmp_query_str, " ", MAX_OPT_ARG_C);*/
+  opt_argv = g_strsplit(query_str, " ", MAX_OPT_ARG_C);
 
   /* Determine the number of arguments. */
   for (opt_argc=0; opt_argv[opt_argc] != NULL; opt_argc++);
 
   dieif( (gst = mg_new(0)) == NULL );
   
-  while ((c = mg_getopt(opt_argc, opt_argv, "adegi:klrmq:s:t:v:xFKLMRST:V:", 
+  while ((c = mg_getopt(opt_argc, opt_argv, "acdegi:klrmq:s:t:v:xFKLMRST:V:", 
 			gst)) != EOF) {
     switch (c) {
       case 'a':
@@ -397,6 +395,10 @@ int QC_fill(char *query_str,
 
       break;
 
+      case 'c':
+        query_command->c_irt_search = TRUE;
+        break;
+
       case 'e':
         query_command->e=1;
       break;
@@ -414,16 +416,21 @@ int QC_fill(char *query_str,
 	  char *hackstr = NULL;
 
           inv_attrs_str = gst->optarg;
-          /* Now a really stupid hard-coded hack to support "pn" being a synonym for "ac,tc,zc,ah" */
-          /* I particularly object to this because it references attributes that should only be 
-             defined in XML - but I don't see a simplier more robust way of doing this hack.
+          /* 
+             Now a really stupid hard-coded hack to support "pn" being a
+             synonym for "ac,tc,zc,ah".  I particularly object to this because
+             it references attributes that should only be defined in XML - but
+             I don't see a simplier more robust way of doing this hack.
              :-( - ottrey 8/12/99 
+
 	     ** removed a memory leak - MB, 1/08/00
+
+             ** removed the use of "ro", added "person" - shane, 2002-01-23
+
 	     */
           if (   strcmp(inv_attrs_str, "pn") == 0 
-	      || strcmp(inv_attrs_str, "ro") == 0) {
-	    wr_malloc( (void **)& hackstr, 24);  /* make a copy */
-	    strcpy(hackstr, "ac,tc,zc,ah");      
+	      || strcmp(inv_attrs_str, "person") == 0) {
+            hackstr = UT_strdup("ac,tc,zc,ah,cn");
 	    inv_attrs_str = hackstr;
           }
           while (*inv_attrs_str) {
@@ -432,7 +439,7 @@ int QC_fill(char *query_str,
 	      /* Unknown attribute encountered. */
 	      char *rep = ca_get_qc_badattr ;
               SK_cd_puts(&(qe->condat), rep);
-	      wr_free(rep);
+	      UT_free(rep);
 
               attr = -1;
               badparerr++;
@@ -449,14 +456,14 @@ int QC_fill(char *query_str,
 	        char *rep = ca_get_qc_fmt_attrnotinv ;
 	        SK_cd_printf(&(qe->condat), rep, 
 		    (DF_get_attribute_aliases())[index]);
-                wr_free(rep);
+                UT_free(rep);
                 badparerr++;
               }
             } 
           } /* while () */
 
 	  if( hackstr != NULL) {
-	    wr_free(hackstr);
+	    UT_free(hackstr);
 	  }
         } /* if () */
       break;
@@ -512,7 +519,7 @@ int QC_fill(char *query_str,
 	      /* Unknown source %s requested. */
 	      char *rep = ca_get_qc_fmt_badsource ;
 	      SK_cd_printf(&(qe->condat), rep, token ); 
-	      wr_free(rep);
+	      UT_free(rep);
 	      
 	      /* XXX error */
 	      badparerr++;
@@ -531,7 +538,7 @@ int QC_fill(char *query_str,
 	      /* Unknown object type encountered */
 	      char *rep = ca_get_qc_badobjtype ;
               SK_cd_puts(&(qe->condat), rep);
-	      wr_free(rep);
+	      UT_free(rep);
               badparerr++;
             }
             else {
@@ -551,7 +558,7 @@ int QC_fill(char *query_str,
 	      /* Unknown object type encountered */
 	      char *rep = ca_get_qc_badobjtype ;
               SK_cd_puts(&(qe->condat), rep);
-	      wr_free(rep);
+	      UT_free(rep);
               badparerr++;
             }
             else {
@@ -602,7 +609,7 @@ int QC_fill(char *query_str,
 	      /* Unknown object type encountered */
 	      char *rep = ca_get_qc_badobjtype ;
               SK_cd_puts(&(qe->condat), rep);
-	      wr_free(rep);
+	      UT_free(rep);
               badparerr++;
             }
             else {
@@ -617,7 +624,7 @@ int QC_fill(char *query_str,
       case 'V':
         if (qe->version != NULL) {
           /* free up the old client info */
-          wr_free(qe->version);
+          UT_free(qe->version);
         }
         
         {
@@ -626,9 +633,7 @@ int QC_fill(char *query_str,
             if(IP_addr_e2b( & (qe->pIP), token) 
                != IP_OK ) {
               /* means it was not an IP -> it was a version */
-              dieif( wr_malloc( (void **)&(qe->version), 
-				strlen(token)+1) != UT_OK);  
-              strcpy(qe->version, token);
+              qe->version = UT_strdup(token);
             }
           }
         }
@@ -651,7 +656,8 @@ int QC_fill(char *query_str,
     }
   }
   /* allocate */
-  dieif( wr_calloc((void **)&(query_command->keys), 1, key_length+1) != UT_OK);  
+  query_command->keys = (char *)UT_calloc(1, key_length+1);
+
   /* copy */
   for (i=gst->optind; i < opt_argc; i++) {
     strcat(query_command->keys, opt_argv[i]);
@@ -689,7 +695,8 @@ int QC_fill(char *query_str,
     }
 
     /* tracing */
-    log_command(tmp_query_str, query_command);
+/*    log_command(tmp_query_str, query_command);*/
+    log_command(query_str, query_command);
 
     /* "keep connection" processing:
        when opening connection, -k may be alone or with a query
@@ -710,8 +717,8 @@ int QC_fill(char *query_str,
   } /* if no error */
 
   /* we don't need this anymore */
-  wr_free(tmp_query_str);
-  wr_free(gst);
+/*  UT_free(tmp_query_str);*/
+  UT_free(gst);
   g_strfreev(opt_argv);
 
   if(synerrflg > 0) { /* severe syntax error. Usage must be printed */
@@ -742,8 +749,7 @@ int QC_fill(char *query_str,
 Query_environ *QC_environ_new(char *ip, int sock) {
   Query_environ *qe;
 
-
-  dieif( wr_calloc((void **)&qe, 1, sizeof(Query_environ)+1 ) != UT_OK);  
+  qe = (Query_environ *)UT_calloc(1, sizeof(Query_environ));
   qe->condat.ip = ip;
   qe->condat.sock = sock;
 
@@ -757,7 +763,7 @@ Query_environ *QC_environ_new(char *ip, int sock) {
       if( strcmp(amrmrulez, "y")==0 ) {
 	qe->sources_list = g_list_append(qe->sources_list, (void *)hdl);
       }
-      free(amrmrulez);
+      UT_free(amrmrulez);
     }
   }
   
@@ -788,7 +794,7 @@ Query_command *QC_create(char *input, Query_environ *qe)
 {
   Query_command *qc;
   /* allocate place for a copy of the input */
-  char *copy = calloc(1,strlen(input)+1); 
+  char *copy = UT_calloc(1,strlen(input)+1); 
   unsigned char *ci, *co;
   int qt;
   /* clean the string from junk - allow only known chars, something like
@@ -797,8 +803,6 @@ Query_command *QC_create(char *input, Query_environ *qe)
      strip leading spaces too
   */
 
-  dieif(copy == NULL);
-
   for(ci = (unsigned char *)input; *ci != 0 && isspace(*ci); ci++) {
     /* EMPTY */
   }
@@ -806,7 +810,7 @@ Query_command *QC_create(char *input, Query_environ *qe)
   for(co = (unsigned char *) copy; *ci != 0; ci++) {
     if( strchr("ABCDEFGHIJKLMNOPQRSTUVWXYZ"     /* only those are allowed */
 	       "abcdefghijklmnopqrstuvwxyz"
-	       "0123456789-_:+=.,@/' \n", *ci) != NULL) {
+	       "0123456789-_:+=.,@/?' \n", *ci) != NULL) {
       *(co++) = *ci;
     }
   }
@@ -819,7 +823,7 @@ Query_command *QC_create(char *input, Query_environ *qe)
   }
 
 
-  dieif( wr_calloc((void **)&qc, 1, sizeof(Query_command)+1) != UT_OK);
+  qc = (Query_command *)UT_calloc(1, sizeof(Query_command));
   
   if ( strlen(copy) == 0) {
     /* An empty query (Ie return) was sent */
@@ -861,7 +865,7 @@ Query_command *QC_create(char *input, Query_environ *qe)
     }
   }
 
-  free(copy);
+  UT_free(copy);
 
   return qc;
 }
