@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.83 $
+  $Revision: 1.94 $
 
   Query instructions (qi).  This is where the queries are executed.
 
@@ -12,7 +12,7 @@
 			accounting support and watchdog cancellation.
   OSs Tested          : Solaris
   ******************/ /******************
-  Copyright (c) 1999                              RIPE NCC
+  Copyright (c) 1999,2000,2001,2002               RIPE NCC
  
   All Rights Reserved
   
@@ -31,29 +31,12 @@
   AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
   ***************************************/
+#include "rip.h"
+
 #include <stdio.h>
 #include <string.h>
 #include <glib.h>
 
-#include "which_keytypes.h"
-#include "query_instructions.h"
-#include "mysql_driver.h"
-#include "rp.h"
-#include "stubs.h"
-#include "constants.h"
-#include "memwrap.h"
-#include "wh_queries.h"
-#include "ud.h"
-
-#include "defs.h"
-
-/*+ String sizes +*/
-#define STR_S   63
-#define STR_M   255
-#define STR_L   1023
-#define STR_XL  4095
-#define STR_XXL 16383
-
 /* the TEMPORARY table is only available in 3.23 or later MySQL */
 #if MYSQL_VERSION_ID >= 32300
 #define USE_TEMPORARY_TABLE 
@@ -211,7 +194,7 @@ int sql_execute_watched(sk_conn_st *condat, SQ_connection_t **sql_connection,
     ottrey
   +html+ </PRE>
   ++++++++++++++++++++++++++++++++++++++*/
-static void create_name_query(char *query_str, const char *sql_query, const char *keys) {
+static void create_name_query(GString *query_str, const char *sql_query, const char *keys) {
   int i;
   /* Allocate stuff - use dynamic strings (initialised to some length) */
   GString *from_clause = g_string_sized_new(STR_L);
@@ -221,15 +204,19 @@ static void create_name_query(char *query_str, const char *sql_query, const char
   /* double quotes " are used in queries to allow querying for 
      names like O'Hara */
 
-  g_string_sprintfa(from_clause, "names N%.2d", 0);
-  g_string_sprintfa(where_clause, "N%.2d.name=\"%s\"", 0, words[0]);
+  if (words[0] != NULL) {
+    g_string_sprintfa(from_clause, "names N%.2d", 0);
+    g_string_sprintfa(where_clause, "N%.2d.name=\"%s\"", 0, words[0]);
 
-  for (i=1; words[i] != NULL; i++) {
-    g_string_sprintfa(from_clause, ", names N%.2d", i);
-    g_string_sprintfa(where_clause, " AND N%.2d.name=\"%s\" AND N00.object_id = N%.2d.object_id", i, words[i], i);
+    for (i=1; words[i] != NULL; i++) {
+      g_string_sprintfa(from_clause, ", names N%.2d", i);
+      g_string_sprintfa(where_clause, 
+          " AND N%.2d.name=\"%s\" AND N00.object_id = N%.2d.object_id", 
+          i, words[i], i);
+    }
   }
 
-  sprintf(query_str, sql_query, from_clause->str, where_clause->str);
+  g_string_sprintf(query_str, sql_query, from_clause->str, where_clause->str);
 
   /* Free up stuff */
   g_strfreev(words);
@@ -259,7 +246,7 @@ static void create_name_query(char *query_str, const char *sql_query, const char
   Author:
     marek
   ++++++++++++++++++++++++++++++++++++++*/
-static int create_asblock_query(char *query_str, 
+static int create_asblock_query(GString *query_str, 
 				const char *sql_query, 
 				const char *keys) {
   char *keycopy = wr_string(keys);
@@ -280,9 +267,9 @@ static int create_asblock_query(char *query_str,
   }
   
   /* now construct the query */
-  sprintf(query_str, sql_query, asnums[0], asnums[1]);
+  g_string_sprintf(query_str, sql_query, asnums[0], asnums[1]);
 
-  wr_free(keycopy);
+  UT_free(keycopy);
   return 0;
 }
 
@@ -300,9 +287,10 @@ static int create_asblock_query(char *query_str,
   Author:
     ottrey.
   ++++++++++++++++++++++++++++++++++++++*/
-static void add_filter(char *query_str, const Query_command *qc) {
+static void add_filter(GString *query_str, const Query_command *qc) 
+{
   unsigned i;
-  int qlen;
+/*  int qlen;*/
   char filter_atom[STR_M];
 
 #if 0
@@ -323,7 +311,7 @@ static void add_filter(char *query_str, const Query_command *qc) {
 
   /* add filters only if any bits are 0 (the number of 1's is < MAX_MAX */
   if (MA_bitcount(qc->object_type_bitmap) != MASK_MAX) { 
-    strcat(query_str, " AND (");
+    g_string_append(query_str, " AND (");
     for (i=0; i < C_END; i++) {
       if (MA_isset(qc->object_type_bitmap, i)) {
         strcpy(filter_atom, "");
@@ -333,13 +321,19 @@ static void add_filter(char *query_str, const Query_command *qc) {
 			   but currently the tables contain values of enums
 			   (C_IN, etc) and not codes
 			*/
-        strcat(query_str, filter_atom);
+        g_string_append(query_str, filter_atom);
       }
     }
-    qlen = strlen(query_str);
+    dieif(query_str->len < 3);  /* this code can only be reached if there is
+                                   at least one object here, meaning this 
+                                   must end with the string "OR ", which we
+                                   then remove */
+    g_string_truncate(query_str, query_str->len - 3);
+    g_string_append_c(query_str, ')');
+/*    qlen = strlen(query_str);
     query_str[qlen-3] = ')';
     query_str[qlen-2] = '\0';
-    query_str[qlen-1] = '\0';
+    query_str[qlen-1] = '\0';*/
   }
 
 #endif
@@ -366,12 +360,15 @@ static void add_filter(char *query_str, const Query_command *qc) {
   +html+ </PRE>
 
   ++++++++++++++++++++++++++++++++++++++*/
-static char *create_query(const Query_t q, const Query_command *qc) {
-  char *result=NULL;
-  char result_buff[STR_XL];
+static char *create_query(const Query_t q, const Query_command *qc) 
+{
+  GString *result_buff;
+  char *result;
   Q_Type_t querytype;
   int addquery = 0; /* controls if the query should be added to the list */
 
+  result_buff = g_string_sized_new(STR_XL);
+
   if (MA_bitcount(qc->inv_attrs_bitmap) > 0) {
     querytype = Q_INVERSE;
   }
@@ -408,10 +405,10 @@ static char *create_query(const Query_t q, const Query_command *qc) {
 	      const char *inet_ntop_ret;
 	      inet_ntop_ret = inet_ntop(AF_INET, &begin, buf, sizeof(buf));
 	      dieif(inet_ntop_ret == NULL);
-	      sprintf(result_buff, q.query, buf);
+	      g_string_sprintf(result_buff, q.query, buf);
 	  } else {
 	      /* otherwise, execute appropriate query on numeric values */
-	      sprintf(result_buff, q.query, begin, end);
+	      g_string_sprintf(result_buff, q.query, begin, end);
 	  }
 	  addquery = 1;
 	}
@@ -429,7 +426,7 @@ static char *create_query(const Query_t q, const Query_command *qc) {
       }
     }
     else {
-      sprintf(result_buff, q.query, qc->keys);
+      g_string_sprintf(result_buff, q.query, qc->keys);
       addquery = 1;
     }
 
@@ -440,13 +437,13 @@ static char *create_query(const Query_t q, const Query_command *qc) {
   }
   
   if( addquery == 1 ) {
-    dieif( wr_malloc((void **)&result, strlen(result_buff)+1) != UT_OK);  
-    strcpy(result, result_buff);
-    return result;
-  } 
-  else {
-    return NULL;
+    result = UT_strdup(result_buff->str);
+  } else {
+    result = NULL;
   }
+  g_string_free(result_buff, TRUE);
+
+  return result;
 } /* create_query() */
 
 /* QI_fast_output() */
@@ -532,7 +529,7 @@ char *QI_fast_output(const char *str)
   g_strfreev(lines);
   
   g_string_append_c(result_buff, '\n');
-  result = strdup(result_buff->str);
+  result = UT_strdup(result_buff->str);
   dieif(result == NULL);
   
   g_string_free(result_buff,/* CONSTCOND */ TRUE);
@@ -570,7 +567,11 @@ char *filter(const char *str) {
   
   for (i=0; filter_names[i] != NULL; i++) {
     for (j=0; lines[j] != NULL; j++) {
-      if (strncmp(filter_names[i], lines[j], strlen(filter_names[i])) == 0) {
+      /* match lines that start with "key-field:" */
+      int filter_name_len = strlen(filter_names[i]);
+      if ((strncmp(filter_names[i], lines[j], filter_name_len) == 0) &&
+          (lines[j][filter_name_len] == ':'))
+      {
 
         g_string_sprintfa(result_buff, "%s\n", lines[j]);
 	passed++;
@@ -601,7 +602,7 @@ char *filter(const char *str) {
   if(passed) {
     g_string_append(result_buff, "\n");
   }
-  result = strdup(result_buff->str);
+  result = UT_strdup(result_buff->str);
   g_string_free(result_buff,/* CONSTCOND */ TRUE);
 
   return result;
@@ -648,22 +649,18 @@ static int write_results(SQ_result_set_t *result,
 	    && AC_credit_isdenied( acc_credit ) == 0
 	    && (row = SQ_row_next(result)) != NULL ) {
       
-      if (  (str = SQ_get_column_string(result, row, 0)) == NULL
-	    || (objt = SQ_get_column_string(result, row, 3)) == NULL )  { 
-	/* handle it somehow ? */
-	die; 
-      }
-      else  { 
-	/* get + add object type */
-	type = atoi(objt);
+      dieif (  (str = SQ_get_column_string(result, row, 0)) == NULL )
+      dieif (  (objt = SQ_get_column_string(result, row, 3)) == NULL );
+
+      /* get + add object type */
+      type = atoi(objt);
 	
-	/* ASP_QI_LAST_DET */
-	ER_dbg_va(FAC_QI, ASP_QI_LAST_DET,
-		  "Retrieved serial id = %d , type = %s", atoi(str), objt);
+      /* ASP_QI_LAST_DET */
+      ER_dbg_va(FAC_QI, ASP_QI_LAST_DET,
+	        "Retrieved serial id = %d , type = %s", atoi(str), objt);
 	
-	wr_free(str);
-	wr_free(objt);
-      }
+      UT_free(str);
+      UT_free(objt);
       
       /* decrement credit for accounting purposes */
       AC_count_object( acc_credit, acl, 
@@ -681,7 +678,7 @@ static int write_results(SQ_result_set_t *result,
         /* The fast output stage */
         if (fast == 1) {
           fasted = QI_fast_output(str);
-          wr_free(str);
+          UT_free(str);
           str = fasted;
         }
 	
@@ -697,11 +694,11 @@ static int write_results(SQ_result_set_t *result,
 
           filtrate = filter(str);
           SK_cd_puts(condat, filtrate);
-          wr_free(filtrate);
+          UT_free(filtrate);
         }
         retrieved_objects++;
       }
-      wr_free(str);
+      UT_free(str);
     }
   }
   
@@ -743,11 +740,11 @@ __report_sql_error(sk_conn_st *condat,
     SK_cd_puts(condat, sql_error_text);
 
     /* next, log this error */
-    ER_perror(FAC_QI, QI_SQLERR,"sql='%s' at %s:%d, tid:%d [%d] %s", 
-		      sql_command,
-		      file, line, tid,
+    ER_perror(FAC_QI, QI_SQLERR, "at %s:%d, tid:%lu [%d] %s sql='%s'", 
+		      file, line, (unsigned long)tid,
 		      SQ_errno(sql_connection), 
-		      SQ_error(sql_connection));
+		      SQ_error(sql_connection),
+		      sql_command);
 }
 
 /* write_objects() */
@@ -800,6 +797,132 @@ write_objects(SQ_connection_t **sql_connection,
   return 0;
 } /* write_objects() */
 
+/* rx_node_has_mnt_irt() */
+/*++++++++++++++++++++++++++++++++++++++
+  See if the node has an "mnt-irt:" attribute.
+
+  condat           connection data for the client
+  sql_connection   connection to the database
+  rx_data          data node returned from the RX module
+  ++++++++++++++++++++++++++++++++++++++*/
+gboolean
+rx_node_has_mnt_irt (sk_conn_st *condat, 
+                     SQ_connection_t *sql_connection, 
+                     int object_id)
+{
+    GString *sql_command;
+    SQ_result_set_t *result_ptr;
+    int sql_error;
+    SQ_row_t *row;
+    long num_irt_ref;
+
+    /* query database for references from this object to an IRT */
+    sql_command = g_string_new("");
+    g_string_sprintf(sql_command, 
+                     "SELECT COUNT(*) FROM mnt_irt WHERE object_id=%d",
+                     object_id);
+    if (SQ_execute_query(sql_connection, sql_command->str, &result_ptr) == -1) {
+        sql_error = SQ_errno(sql_connection);
+        report_sql_error(condat, sql_connection, sql_command->str);
+        g_string_free(sql_command, TRUE);
+        return FALSE;
+    }
+    g_string_free(sql_command, TRUE);
+
+    /* get number of references */
+    row = SQ_row_next(result_ptr);
+    if (row == NULL) {
+        /* if the object is deleted after we have gotten a reference to it,
+           the lookup will fail - in such a case treat it the same as 
+           an object without any mnt_irt attributes */
+        num_irt_ref = 0;
+    } else {
+        if (SQ_get_column_int(result_ptr, row, 0, &num_irt_ref) != 0) {
+            ER_perror(FAC_QI, QI_SQLERR, "bad column at %s:%d", 
+                      __FILE__, __LINE__);
+            num_irt_ref = 0;
+        }
+    }
+    SQ_free_result(result_ptr);
+
+    /* return appropriate result */
+    if (num_irt_ref > 0) {
+        return TRUE;
+    } else {
+        return FALSE;
+    }
+}
+
+
+/* mnt_irt_filter() */
+/*++++++++++++++++++++++++++++++++++++++
+  Find the most specific entry in the list of network nodes passed
+  and remove all entries but that one from the list.
+
+  condat           connection data for the client
+  sql_connection   connection to the database
+  datlist          list of data from the radix tree (nodes of *rx_datcpy_t)
+
+  Note:  The "-c" query flag specifies that the most specific inetnum or
+         inet6num with an "mnt-irt:" attribute should be returned.  
+         
+         To do this, we get a list of encompassing networks by sending the 
+         same options to the RP/RX module that the '-L' query does.  Then
+         we call this function, mnt_irt_filter(), and search the list for 
+         the first entry that has an "mnt-irt:" attribute, by looking in 
+         the "mnt_irt" table in MySQL for a reference (see the 
+         rx_node_has_mnt_irt() for details).
+
+         If a reference is found, the list is replaced with a list 
+         containing the single entry.  If no reference is found, the list
+         is deleted.
+  ++++++++++++++++++++++++++++++++++++++*/
+void 
+mnt_irt_filter (sk_conn_st *condat, 
+                SQ_connection_t *sql_connection, 
+                GList **datlist)
+{
+    GList *p;
+    GList *new_datlist;
+    rx_datcpy_t *rx_data;
+    int object_id;
+
+    /* empty datlist */
+    new_datlist = NULL;
+
+    /* search for node with "mnt-irt:" attribute */
+    p = g_list_last(*datlist);
+    while ((new_datlist == NULL) && (p != NULL)) {
+        /* grab the data for this node */
+        rx_data = (rx_datcpy_t *)p->data;
+        object_id = rx_data->leafcpy.data_key;
+
+        /* see if this is the node we are looking for */
+        if (rx_node_has_mnt_irt(condat, sql_connection, object_id)) {
+            /* use this entry */
+            new_datlist = g_list_append(NULL, rx_data);
+        } 
+        else {
+            /* free memory */
+            UT_free(rx_data->leafcpy.data_ptr);
+        }
+
+        p = g_list_previous(p);
+    }
+
+    /* free our old datlist */
+    while (p != NULL) {
+        rx_data = (rx_datcpy_t *)p->data;
+        UT_free(rx_data->leafcpy.data_ptr);
+        p = g_list_previous(p);
+    }
+    wr_clear_list(datlist);
+
+    /* use our new datlist */
+    *datlist = new_datlist;
+}
+
+
 /* insert_radix_serials() */
 /*++++++++++++++++++++++++++++++++++++++
   Insert the radix serial numbers into a temporary table in the database.
@@ -824,8 +947,8 @@ write_objects(SQ_connection_t **sql_connection,
 static int insert_radix_serials(sk_conn_st *condat,
 				 SQ_connection_t *sql_connection, 
 				 char *id_table, GList *datlist) {
-  GList    *qitem;
-  char sql_command[STR_XL];
+  GList *qitem;
+  GString *sql_command;
   int serial;
   int sql_error;
 
@@ -838,14 +961,17 @@ static int insert_radix_serials(sk_conn_st *condat,
     /* don't bother to insert values into our temporary table */
     /* if we've lost the client connection */
     if ((condat->rtc == 0) && !sql_error) {
-        sprintf(sql_command, "INSERT INTO %s values (%d)", id_table, serial);
-        if (SQ_execute_query(sql_connection, sql_command, NULL) == -1) {
+        sql_command = g_string_sized_new(STR_S);
+        g_string_sprintf(sql_command, 
+                         "INSERT INTO %s values (%d)", id_table, serial);
+        if (SQ_execute_query(sql_connection, sql_command->str, NULL) == -1) {
 	    sql_error = SQ_errno(sql_connection);
-	    report_sql_error(condat, sql_connection, sql_command);
+	    report_sql_error(condat, sql_connection, sql_command->str);
 	}
+        g_string_free(sql_command, TRUE);
     }
 
-    wr_free(datcpy->leafcpy.data_ptr);
+    UT_free(datcpy->leafcpy.data_ptr);
   }
 
   wr_clear_list( &datlist );
@@ -888,7 +1014,7 @@ static void write_radix_immediate(GList *datlist,
     SK_cd_puts(condat, datcpy->leafcpy.data_ptr );
     SK_cd_puts(condat, "\n");
     
-    wr_free(datcpy->leafcpy.data_ptr);
+    UT_free(datcpy->leafcpy.data_ptr);
     
     AC_count_object(acc_credit, acl, 0 /* public object (private=0) */ );
 
@@ -923,6 +1049,8 @@ static int map_qc2rx(Query_instruction *qi, const Query_command *qc) {
   int allflags = (qc->L == 1) + (qc->M == 1) + (qc->l == 1) 
                + (qc->m == 1) + (qc->x == 1);
 
+  allflags += qc->c_irt_search;
+
   qi->rx_keys = qc->keys;
 
   /* only one option can be active at a time */
@@ -939,6 +1067,10 @@ static int map_qc2rx(Query_instruction *qi, const Query_command *qc) {
       qi->rx_srch_mode = RX_SRCH_EXLESS;
       qi->rx_par_a = 0;
   }
+  else if (qc->c_irt_search) {
+      qi->rx_srch_mode = RX_SRCH_LESS;
+      qi->rx_par_a = RX_ALL_DEPTHS;
+  }
   else if ( qc->L == 1 ) {
       qi->rx_srch_mode = RX_SRCH_LESS;
       qi->rx_par_a = RX_ALL_DEPTHS;
@@ -1013,21 +1145,21 @@ void run_referral(Query_environ *qe,
     /* Referral timeout */
     rep = ca_get_qi_ref_tmout ;
     SK_cd_puts(&(qe->condat), rep);
-    wr_free(rep);
+    UT_free(rep);
     break;
 	    
   case SK_BADHOST:
     /* Referral host not found */
     rep = ca_get_qi_ref_badhost ;
     SK_cd_puts(&(qe->condat), rep);
-    wr_free(rep);
+    UT_free(rep);
     break;
 
   case SK_CONNECT:
     /* Referral host not responding */
     rep = ca_get_qi_ref_hostnottresp ;
     SK_cd_puts(&(qe->condat), rep);
-    wr_free(rep);
+    UT_free(rep);
     break;
 
   case SK_BIND:
@@ -1039,7 +1171,7 @@ void run_referral(Query_environ *qe,
     /* Referral reply line limit exceeded */
     rep = ca_get_qi_ref_overmaxlin ;
     SK_cd_puts(&(qe->condat), rep);
-    wr_free(rep);
+    UT_free(rep);
     break;
     
   default: /* any other errors ? */
@@ -1092,7 +1224,7 @@ void qi_prep_run_refer(char *domain,
     long ref_type;
     long ref_port;
     char *ref_host; 
-    char querystr[STR_L];
+    GString *querystr;
 
     /* get values from SQL query */
     err = SQ_get_column_int(result, row, 0, &ref_type);
@@ -1102,29 +1234,29 @@ void qi_prep_run_refer(char *domain,
     ref_host = SQ_get_column_string(result, row, 2);
     dieif(ref_host == NULL);
       
-    strcpy(querystr,"");
+    querystr = g_string_sized_new(STR_L);
       
     /* put -r if the reftype is RIPE and -r or -i were used */
     if( (ref_type == RF_RIPE) 
 	&& ( Query[qi->queryindex].querytype == Q_INVERSE       
 	     || qis->recursive > 0  )   ) 
     {
-	strcpy(querystr,"-r ");
+        g_string_append(querystr, "-r ");
     }
       
     /* prepend with -Vversion,IP for type CLIENTADDRESS */
     if( ref_type == RF_CLIENTADDRESS ) {
-	snprintf(querystr,sizeof(querystr),"-V%s,%s ",VERSION, qe->condat.ip);
+        g_string_sprintf(querystr, "-V%s,%s ", VERSION, qe->condat.ip);
     }
 
       
     /* now set the search term - set to the stripped down version 
        for inverse query, full-length otherwise */
     if( Query[qi->queryindex].querytype == Q_INVERSE ) {
-	strcat(querystr, domain);
+        g_string_append(querystr, domain);
     }
     else {
-	strcat(querystr, qis->qc->keys);
+        g_string_append(querystr, qis->qc->keys);
     }
 
     {
@@ -1135,22 +1267,23 @@ void qi_prep_run_refer(char *domain,
 		     sourcename, 
 		     ref_host, ref_port,
 		     sourcename );
-	wr_free(rep);
+	UT_free(rep);
     }
 	    
     /* do the referral */
     ER_dbg_va(FAC_QI, ASP_QI_REF_GEN, "referral host is %s", ref_host); 
       
-    run_referral( qe, ref_host, ref_port, querystr);
+    run_referral( qe, ref_host, ref_port, querystr->str);
       
     { /* End of referred query result */
 	char *rep = ca_get_qi_reftrailer ; 
 	SK_cd_puts(&(qe->condat), rep);
-	wr_free(rep);
+	UT_free(rep);
     }
     SK_cd_puts(&(qe->condat), "\n");
     
-    wr_free(ref_host);
+    g_string_free(querystr, TRUE);
+    UT_free(ref_host);
 }
 
 
@@ -1197,26 +1330,33 @@ qi_collect_domain(char *sourcename,
   char *dot = domain;
   int subcount = 0;
   int foundcount = 0;
+  GString *sql_command;
 
   /* we MUST NOT have a diconnection from the server here */
   dieif(qe->condat.rtc != 0);
 
+  /* create a string for our queries */
+  sql_command = g_string_sized_new(STR_XL);
+
   /* while nothing found and still some pieces of the name left */
   while( dot != NULL && subcount == 0 ) { 
     int refcount = 0;
     SQ_row_t *row;
     SQ_result_set_t *result_referrals = NULL;
-    char sql_command[STR_XL];
 
     ER_dbg_va(FAC_QI, ASP_QI_REF_DET, "run_referral: checking %s", dot);
 
     /* domain lookup -- query into the _S table */
-    sprintf(sql_command, "INSERT INTO %s SELECT object_id FROM domain WHERE domain = '%s'", sub_table, dot);
-    
-    if( SQ_execute_query(sql_connection, sql_command, NULL) == -1) {
+    g_string_sprintf(sql_command, 
+                     "INSERT INTO %s SELECT object_id "
+                         "FROM domain "
+                         "WHERE domain = '%s'",
+                     sub_table, dot);
+    if (SQ_execute_query(sql_connection, sql_command->str, NULL) != 0) {
         *sql_error = SQ_errno(sql_connection);
-	report_sql_error(&qe->condat, sql_connection, sql_command);
-	return 0;
+	report_sql_error(&qe->condat, sql_connection, sql_command->str);
+        foundcount = 0;
+        goto exit_qi_collect_domain;
     }
     subcount = SQ_get_affected_rows(sql_connection); 
 
@@ -1225,13 +1365,19 @@ qi_collect_domain(char *sourcename,
 	/* referral check. Always done except for -R and INVERSE queries */  
 	if( qis->qc->R == 0 && 
 	    Query[qi->queryindex].querytype != Q_INVERSE ) {
-	    sprintf(sql_command, "SELECT type, port, host FROM %s ID, refer WHERE ID.id = refer.object_id", sub_table);
-	    if( SQ_execute_query(sql_connection, sql_command, 
+            g_string_sprintf(sql_command, 
+                             "SELECT type, port, host "
+                                 "FROM %s ID, refer "
+                                 "WHERE ID.id = refer.object_id", 
+                             sub_table);
+
+	    if( SQ_execute_query(sql_connection, sql_command->str, 
 				    &result_referrals) == -1) 
             {
 	        *sql_error = SQ_errno(sql_connection);
-	        report_sql_error(&qe->condat, sql_connection, sql_command);
-		return 0;
+	        report_sql_error(&qe->condat, sql_connection, sql_command->str);
+                foundcount = 0;
+                goto exit_qi_collect_domain;
             }
 	    refcount = SQ_num_rows(result_referrals);
 	}
@@ -1258,12 +1404,17 @@ qi_collect_domain(char *sourcename,
 	       * a 'stripped' domain name -> return no result and quit
 	    */
 	    if( dot == domain ) {
-		sprintf(sql_command, "INSERT INTO %s SELECT id FROM %s", 
-			id_table, sub_table);
-		if (SQ_execute_query(sql_connection, sql_command, NULL) == -1) {
+		g_string_sprintf(sql_command, 
+                                 "INSERT INTO %s SELECT id FROM %s", 
+			         id_table, sub_table);
+		if (SQ_execute_query(sql_connection, 
+                                     sql_command->str, NULL) == -1) 
+                {
 		    *sql_error = SQ_errno(sql_connection);
-	            report_sql_error(&qe->condat, sql_connection, sql_command);
-		    return 0;
+	            report_sql_error(&qe->condat, sql_connection, 
+                                     sql_command->str);
+                    foundcount = 0;
+                    goto exit_qi_collect_domain;
 		}
 		foundcount = SQ_get_affected_rows(sql_connection); 
 	    }
@@ -1280,9 +1431,15 @@ qi_collect_domain(char *sourcename,
       dot++;
     }
   }
+
+/* unified success/failure exit point to perform cleanup */
+exit_qi_collect_domain:
+
+  /* free the string we used for queries */
+  g_string_free(sql_command, TRUE);
     
   return foundcount;
-} /* check_domain */
+} /* qi_collect_domain */
 
 
 /* add_ref_name */
@@ -1307,29 +1464,37 @@ add_ref_name(SQ_connection_t *sql_connection,
 	     sk_conn_st *condat
 	     )
 {
+  int error;
+
+  error = 0;
+
   /* construct the query, allow zero-length list */
   if( strlen(allnames) > 0 ) {
-    char final_query[STR_XL];
-    char select_query[STR_XL];
+    GString *final_query;
+    GString *select_query;
+
+    final_query = g_string_sized_new(STR_XL);
+    select_query = g_string_sized_new(STR_XL);
 
     create_name_query(select_query, "SELECT N00.object_id FROM %s WHERE %s "
 		      "AND N00.object_type != 100 AND N00.thread_id = 0", 
 		      allnames);
     
-    sprintf(final_query, "INSERT INTO %s %s",
-	    rectable,
-	    select_query);
+    g_string_sprintf(final_query, "INSERT INTO %s %s",
+	             rectable, select_query->str);
 
     allnames[0]=0;
     
-    if (SQ_execute_query(sql_connection, final_query, NULL) == -1 ) {
-        report_sql_error(condat, sql_connection, final_query);
-        return SQ_errno(sql_connection);
+    if (SQ_execute_query(sql_connection, final_query->str, NULL) == -1 ) {
+        report_sql_error(condat, sql_connection, final_query->str);
+        error = SQ_errno(sql_connection);
     }
 
+    g_string_free(select_query, TRUE);
+    g_string_free(final_query, TRUE);
   }
 
-  return 0;
+  return error;
 }/* add_ref_name */
 
 
@@ -1381,9 +1546,9 @@ qi_collect_ids(ca_dbSource_t *dbhdl,
   Query_instruction **ins=NULL;
   int i;
   int  count, errors=0;
-  char sql_command[STR_XL];
+  GString *sql_command;
   er_ret_t err;
-  char sub_table[32];
+  char sub_table[64];
   int limit ;
              /* a limit on the max number of objects to be returned
 		from a single search. For some queries the object types
@@ -1394,6 +1559,9 @@ qi_collect_ids(ca_dbSource_t *dbhdl,
   char limit_str[32];
   int sql_error;
 
+  /* use a nice resizing GString for our command */
+  sql_command = g_string_sized_new(STR_XL);
+
   if( (limit = AC_get_higher_limit(acc_credit,acl)) == -1) {
     strcpy(limit_str,"");
   } else {
@@ -1410,17 +1578,18 @@ qi_collect_ids(ca_dbSource_t *dbhdl,
    * update: this can't ever happen with TEMPORARY tables, but we're going to
    *         check for it anyway  - shane
    */
-  sprintf(sql_command, "DROP TABLE IF EXISTS %s", sub_table);
-  if( SQ_execute_query(*sql_connection, sql_command, NULL) == -1 ) {
-      report_sql_error(&qe->condat, *sql_connection, sql_command);
+  g_string_sprintf(sql_command, "DROP TABLE IF EXISTS %s", sub_table);
+  if( SQ_execute_query(*sql_connection, sql_command->str, NULL) == -1 ) {
+      report_sql_error(&qe->condat, *sql_connection, sql_command->str);
       return SQ_errno(*sql_connection);
   }
 
   /* create a table for special subqueries (domain only for now) */
-  sprintf(sql_command, "CREATE " TEMPORARY " TABLE %s ( id int ) TYPE=HEAP", 
-    sub_table);
-  if( SQ_execute_query(*sql_connection, sql_command, NULL) == -1 ) {
-      report_sql_error(&qe->condat, *sql_connection, sql_command);
+  g_string_sprintf(sql_command, 
+                   "CREATE " TEMPORARY " TABLE %s ( id int ) TYPE=HEAP", 
+                   sub_table);
+  if( SQ_execute_query(*sql_connection, sql_command->str, NULL) == -1 ) {
+      report_sql_error(&qe->condat, *sql_connection, sql_command->str);
       return SQ_errno(*sql_connection);
   }
   
@@ -1437,6 +1606,7 @@ qi_collect_ids(ca_dbSource_t *dbhdl,
 
     switch ( qi->search_type ) {
     case R_SQL:
+      count = 0;
       if ( qi->query_str != NULL ) {
 
 	/* handle special cases first */
@@ -1456,14 +1626,14 @@ qi_collect_ids(ca_dbSource_t *dbhdl,
 	else {
 	  /* any other class of query */
 
-	  sprintf(sql_command, "INSERT INTO %s %s %s", 
+	  g_string_sprintf(sql_command, "INSERT INTO %s %s %s", 
 		  id_table, qi->query_str, limit_str);
 
 	  if(sql_execute_watched( &(qe->condat), sql_connection, 
-				  sql_command, NULL) == -1 ) {
+				  sql_command->str, NULL) == -1 ) {
 	    errors++;
 	    sql_error = SQ_errno(*sql_connection);
-	    report_sql_error(&qe->condat, *sql_connection, sql_command);
+	    report_sql_error(&qe->condat, *sql_connection, sql_command->str);
 	  }
 	  count = SQ_get_affected_rows(*sql_connection);
 	} /* not DN */
@@ -1509,12 +1679,15 @@ qi_collect_ids(ca_dbSource_t *dbhdl,
   } /* for <every instruction> */
 
   /* Now drop the _S table */
-  sprintf(sql_command, "DROP TABLE IF EXISTS %s", sub_table);
-  if(SQ_execute_query(*sql_connection, sql_command, NULL) == -1 ) {
-      report_sql_error(&qe->condat, *sql_connection, sql_command);
-      return SQ_errno(*sql_connection);
+  g_string_sprintf(sql_command, "DROP TABLE IF EXISTS %s", sub_table);
+  if(SQ_execute_query(*sql_connection, sql_command->str, NULL) == -1 ) {
+      report_sql_error(&qe->condat, *sql_connection, sql_command->str);
+      sql_error = SQ_errno(*sql_connection);
   }
 
+  /* free our command buffer */
+  g_string_free(sql_command, TRUE);
+
   /* return success */
   return sql_error;
 }
@@ -1548,7 +1721,7 @@ qi_fetch_references(SQ_connection_t **sql_connection,
 		    acl_st *acl
 		    )
 {
-    char rec_table[32];
+    char rec_table[64];
     SQ_result_set_t *result = NULL;
     SQ_row_t *row;
     int thisid = 0;
@@ -1665,8 +1838,8 @@ qi_fetch_references(SQ_connection_t **sql_connection,
 	strcat(allnames, name);
 	strcat(allnames, " ");
 	oldid = thisid;
-	wr_free(id);
-	wr_free(name);
+	UT_free(id);
+	UT_free(name);
       }
       /* also do the last name */
       if (!sql_error) {
@@ -1742,7 +1915,7 @@ er_ret_t QI_execute(ca_dbSource_t *dbhdl,
   char *dbpass = ca_get_srcdbpassword(dbhdl);
   char *srcnam;
   unsigned dbport = ca_get_srcdbport(dbhdl);
-  char id_table[STR_S];
+  char id_table[64];
   char sql_command[STR_XL];
   GList *datlist=NULL;
   SQ_connection_t *sql_connection=NULL;
@@ -1751,24 +1924,24 @@ er_ret_t QI_execute(ca_dbSource_t *dbhdl,
   sql_connection = SQ_get_connection( dbhost, dbport,
 				      dbname, dbuser, dbpass );
   /* free parameters when done */
-  wr_free(dbhost);
-  wr_free(dbuser);
-  wr_free(dbpass);
+  UT_free(dbhost);
+  UT_free(dbuser);
+  UT_free(dbpass);
 
   /* return error if occurred */
   if (sql_connection == NULL) {
     ER_perror(FAC_QI, QI_CANTDB," database='%s' [%d] %s", 
 	      dbname, SQ_errno(sql_connection), SQ_error(sql_connection));
-    wr_free(dbname);
+    UT_free(dbname);
     return QI_CANTDB;
   }
-  wr_free(dbname);
+  UT_free(dbname);
 
   /* from here on out, we use the sql_error flag to verify our 
      connection to the SQL database is still good */
   sql_error = 0;
 
-  sprintf(id_table, "ID_%ld_%d",   mysql_thread_id(sql_connection),
+  sprintf(id_table, "ID_%lu_%u",   mysql_thread_id(sql_connection),
 	  pthread_self());
 
   /* see if there was a leftover table from a crashed session 
@@ -1795,7 +1968,7 @@ er_ret_t QI_execute(ca_dbSource_t *dbhdl,
       srcnam = ca_get_srcname(dbhdl);
       sql_error = qi_collect_ids(dbhdl, srcnam, &sql_connection, qis, qe, 
                                  id_table, &datlist, acc_credit, acl);
-      wr_free(srcnam);
+      UT_free(srcnam);
   }
 
   /* post-processing */
@@ -1806,6 +1979,11 @@ er_ret_t QI_execute(ca_dbSource_t *dbhdl,
 
       /* add radix results (only if -K is not active and still connected) */
       if (qe->condat.rtc == 0) {
+
+          if (qis->qc->c_irt_search) {
+              mnt_irt_filter(&(qe->condat), sql_connection, &datlist);
+          }
+
           sql_error = insert_radix_serials(&(qe->condat), 
 	                                   sql_connection, 
 	                                   id_table, 
@@ -1875,9 +2053,9 @@ er_ret_t QI_execute(ca_dbSource_t *dbhdl,
 static void instruction_free(Query_instruction *qi) {
   if (qi != NULL) {
     if (qi->query_str != NULL) {
-      wr_free(qi->query_str);
+      UT_free(qi->query_str);
     }
-    wr_free(qi);
+    UT_free(qi);
   }
 } /* instruction_free() */
 
@@ -1901,7 +2079,7 @@ void QI_free(Query_instructions *qis) {
   } 
 
   if (qis != NULL) {
-    wr_free(qis);
+    UT_free(qis);
   }
 
 } /* QI_free() */
@@ -1977,7 +2155,7 @@ Query_instructions *QI_new(const Query_command *qc, const Query_environ *qe) {
   int i;
   char *query_str;
 
-  dieif(wr_calloc( (void **) & qis, 1, sizeof(Query_instructions)) != UT_OK);
+  qis = (Query_instructions *)UT_calloc(1, sizeof(Query_instructions));
 
   qis->filtered = qc->filtered;
   qis->fast = qc->fast;
@@ -1990,7 +2168,7 @@ Query_instructions *QI_new(const Query_command *qc, const Query_environ *qe) {
     /* If a valid query. */
     if ( valid_query(qc, Query[i]) == 1) {
 
-      dieif( wr_calloc((void **) &qi, 1, sizeof(Query_instruction)) != UT_OK);
+      qi = (Query_instruction *)UT_calloc(1, sizeof(Query_instruction));
 
       qi->queryindex = i;
 
@@ -2006,6 +2184,12 @@ Query_instructions *QI_new(const Query_command *qc, const Query_environ *qe) {
       }
       /* Radix Query */
       else if (Query[i].refer == R_RADIX) {
+
+         /* no inverse queries should use the RADIX tree */
+        if (MA_bitcount(qc->inv_attrs_bitmap) > 0) {
+            continue;
+        }
+
         qi->search_type = R_RADIX;
 	
         if (map_qc2rx(qi, qc) == 1) {
@@ -2028,7 +2212,7 @@ Query_instructions *QI_new(const Query_command *qc, const Query_environ *qe) {
 	  
           if ( found ) {
             /* Discard the Query Instruction */
-            wr_free(qi);
+            UT_free(qi);
           } 
           else {
             /* Add the query_instruction to the array */
@@ -2049,7 +2233,7 @@ Query_instructions *QI_new(const Query_command *qc, const Query_environ *qe) {
       char *descrstr = QI_queries_to_string(qis);
 
       ER_dbg_va(FAC_QI, ASP_QI_COLL_GEN, "Queries: %s", descrstr );
-      wr_free( descrstr );
+      UT_free( descrstr );
   }
 
   return qis;
@@ -2076,16 +2260,16 @@ char *QI_queries_to_string(Query_instructions *qis)
 {
    Query_instruction *qi;
    int i;
-   char *resstr = NULL;
+   char *resstr;
 
-   dieif( wr_realloc((void **)&resstr, 2 ) != UT_OK);
+   resstr = (char *)UT_malloc(2);
    strcpy(resstr, "{");
 
    for( i = 0; ( qi=qis->instruction[i] ) != NULL;  i++ ) {
        char *descr = Query[qi->queryindex].descr;
        int oldres = strlen( resstr );
        
-       dieif( wr_realloc((void **)&resstr, oldres+strlen(descr)+2) != UT_OK);
+       resstr = (char *)UT_realloc(resstr, oldres+strlen(descr)+2);
        strcat(resstr, descr);
        strcat(resstr, ",");
    }
@@ -2094,8 +2278,7 @@ char *QI_queries_to_string(Query_instructions *qis)
        resstr[strlen(resstr)-1] = 0;
    }
 
-   dieif( wr_realloc((void **)&resstr, strlen( resstr ) + 2 ) 
-	  != UT_OK);
+   resstr = (char *)UT_realloc(resstr, strlen(resstr) + 2);
    strcat(resstr, "}");
    
    return resstr;
