@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.9 $
+  $Revision: 1.11 $
 
   Radix payload (rp) - user level functions for storing data in radix trees
 
@@ -10,7 +10,7 @@
   Design and implementation by: Marek Bukowy
   
   ******************/ /******************
-  Copyright (c) 1999,2000,2001                    RIPE NCC
+  Copyright (c) 1999,2000,2001,2002               RIPE NCC
  
   All Rights Reserved
   
@@ -60,7 +60,7 @@ int RP_attr2spc(rp_attr_t type, ip_space_t space)
 
 
 er_ret_t
-RP_asc2uni(char       *astr,       /*+ string prefix/range/IP/inaddr +*/
+RP_asc2uni(const char *astr,       /*+ string prefix/range/IP/inaddr +*/
 	   rp_attr_t  attr,
 	   rp_uni_t   *uni)        /* destination pointer */
 {
@@ -100,7 +100,7 @@ RP_asc2uni(char       *astr,       /*+ string prefix/range/IP/inaddr +*/
 /* Function to fill data for radix tree */
 /* returns error if string is not valid, also for reverse domains */
 er_ret_t
-RP_asc2pack(rp_upd_pack_t *pack, rp_attr_t type, char *string)
+RP_asc2pack(rp_upd_pack_t *pack, rp_attr_t type, const char *string)
 {
   er_ret_t err;
 
@@ -162,53 +162,65 @@ void rp_make_short(rp_upd_pack_t *pack, char **ptr, unsigned *len)
 }
 
 /***************** set the values in rx_*_data thingies ***************/
-void RP_pack_set_orig(rp_attr_t  attr, rp_upd_pack_t *pack, char *origin)
+er_ret_t RP_pack_set_orig(rp_attr_t  attr, rp_upd_pack_t *pack, const char *origin)
 {
   pack->d.origin = origin;
+  return(IP_OK);
   /* ignore attr */
 }
 
 /* those are just interfacing to 
  * functions to convert to IP binary format and retain raw values 
  */
-void RP_pack_set_type(rp_attr_t  attr, rp_upd_pack_t *pack)
+er_ret_t RP_pack_set_type(rp_attr_t  attr, rp_upd_pack_t *pack)
 {
   pack->type = attr;
   pack->uni.fam = RP_attr2fam( attr );
+  return(IP_OK);
 }
 
-void RP_pack_set_pref4(rp_attr_t  attr, char *avalue, rp_upd_pack_t *pack,
+er_ret_t RP_pack_set_pref4(rp_attr_t  attr, const char *avalue, rp_upd_pack_t *pack,
 		       unsigned *prefix, unsigned *prefix_length)
 {
-  IP_pref_a2v4(avalue, &(pack->uni.u.rt), prefix, prefix_length);
-  pack->uni.space = IP_V4;
-  RP_pack_set_type(attr, pack);
+er_ret_t ret;
+  if((ret = IP_pref_a2v4(avalue, &(pack->uni.u.rt), prefix, prefix_length)) == IP_OK){
+    pack->uni.space = IP_V4;
+    RP_pack_set_type(attr, pack);
+  }
+  return(ret);
 }
 
-void RP_pack_set_revd(rp_attr_t  attr, char *avalue, rp_upd_pack_t *pack)
+er_ret_t RP_pack_set_revd(rp_attr_t  attr, const char *avalue, rp_upd_pack_t *pack)
 {
   dieif(IP_revd_a2b(&(pack->uni.u.rt), avalue) != IP_OK); /* assuming correctness checked */
   pack->d.domain = avalue;
   pack->uni.space = IP_pref_b2_space( &(pack->uni.u.rt) );
   RP_pack_set_type(attr, pack);
+  return(IP_OK);
 }
 
 
-void RP_pack_set_pref6(rp_attr_t  attr, char *avalue, rp_upd_pack_t *pack,
+er_ret_t RP_pack_set_pref6(rp_attr_t  attr, const char *avalue, rp_upd_pack_t *pack,
 		       ip_v6word_t *high, ip_v6word_t *low, unsigned *prefix_length)
 {
-  IP_pref_a2v6(avalue, &(pack->uni.u.rt), high, low, prefix_length);
-  pack->uni.space = IP_V6;
-  RP_pack_set_type(attr, pack);
+er_ret_t ret;
+  if((ret = IP_pref_a2v6(avalue, &(pack->uni.u.rt), high, low, prefix_length)) == IP_OK){
+    pack->uni.space = IP_V6;
+    RP_pack_set_type(attr, pack);
+  }
+  return(ret);
 }
 
-void RP_pack_set_rang(rp_attr_t  attr, char *avalue, rp_upd_pack_t *pack,
+er_ret_t RP_pack_set_rang(rp_attr_t  attr, const char *avalue, rp_upd_pack_t *pack,
 		      unsigned *begin_in, unsigned *end_in)
 {
-  IP_rang_a2v4(avalue, (ip_range_t *) &(pack->uni.u.in),
-	       begin_in, end_in);
-  pack->uni.space = IP_V4;
-  RP_pack_set_type(attr, pack);
+er_ret_t ret;
+  if((ret = IP_rang_a2v4(avalue, (ip_range_t *) &(pack->uni.u.in),
+	       begin_in, end_in)) == IP_OK) {
+    pack->uni.space = IP_V4;
+    RP_pack_set_type(attr, pack);
+  }
+  return(ret);
 }
 
 
