@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.6 $
+  $Revision: 1.8 $
 
   Socket module - cd_watchdog.c - Socket watchdog - when activated, checks the
                                   socket for new data and discards it. If the
@@ -16,7 +16,7 @@
   marek  (December 2000) Modified watchdog deactivation - 
                          replaced signals by pthread cancellation.
   ******************/ /******************
-  Copyright (c) 1999, 2000, 2001                     RIPE NCC
+  Copyright (c) 1999,2000,2001,2002                     RIPE NCC
  
   All Rights Reserved
   
@@ -320,8 +320,18 @@ SK_watchstart(sk_conn_st *condat)
   pthread_mutex_init( & condat->watchmutex, NULL );
   pthread_mutex_lock( & condat->watchmutex ); 
 
+/* 
+   Linux doesn't seem to like getting signals in select(), which isn't
+   technically allowed by POSIX.  The workaround in this case is simply
+   to not create a watchdog for Linux.  This is probably acceptable
+   because we will be changing the query path to perform queries in small
+   chunks, so if a disconnect occurs it won't consume a lot of database
+   resources in any case, even without a watchdog.
+ */
+#ifndef __linux__
   /* NOT DETACHED! */
   pthread_create(&condat->watchdog, &attr, sk_watchdog, (void *) condat );
+#endif /* __linux__ */
 
   dieif(pthread_attr_destroy(&attr) != 0);
   
