@@ -1,5 +1,5 @@
 /***************************************
-  $Revision: 1.54 $
+  $Revision: 1.62 $
 
   Example code: A server for a client to connect to.
 
@@ -21,7 +21,7 @@
         ottrey (08/03/1999) Modified.
         joao   (22/06/1999) Modified.
   ******************/ /******************
-  Copyright (c) 1999                              RIPE NCC
+  Copyright (c) 1999,2000,2001,2002                  RIPE NCC
  
   All Rights Reserved
   
@@ -41,6 +41,8 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  ***************************************/
 
+#include "rip.h"
+
 #include <ctype.h>
 
 #include <sys/types.h>
@@ -48,44 +50,10 @@
 #include <sys/wait.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
-
-#include "thread.h"
-#include "rxroutines.h"
-#include "sk.h"
-/*
-#include "objects.h"
-*/
-#include "constants.h"
-
-#include "ca_configFns.h"
-#include "ca_dictionary.h"
-#include "ca_macros.h"
-#include "ca_srcAttribs.h"
-
-#include "mysql_driver.h"
-#include "access_control.h"
-#include "ud.h"
-#include "ud_tr.h"
-#include "server.h"
-
-#include "rp.h"
-#include "memwrap.h"
-
-#include "ta.h"
-
-#include "protocol_whois.h"
-#include "protocol_mirror.h"
-#include "protocol_config.h"
-
-/*+ String sizes +*/
-#define STR_S   63
-#define STR_M   255
-#define STR_L   1023
-#define STR_XL  4095
-#define STR_XXL 16383
-
-/* Storage for descriptors of the read side of the pipe */
-int sv_lockfd[MAX_LOCKS];
+#include <errno.h>
+#include <unistd.h>
+#include <string.h>
+#include <sys/poll.h>
 
 /* Listening sockets */
 int SV_whois_sock;
@@ -96,9 +64,17 @@ int SV_mirror_sock;
 #define MAX_SOURCES 100
 int SV_update_sock[MAX_SOURCES];
 
+/* when the process shuts down, it writes to this file descriptor */
+int SV_shutdown_send_fd = -1;
+
+/* server threads should check this file descriptor, and terminate 
+   as soon as possible if data becomes available on it */
+int SV_shutdown_recv_fd = -1;
 
 /*+ Server starting time +*/
 time_t SV_starttime;
+/* the filename where we store the PID of the server */
+char *SV_pidfile;
 
 /* Logging results */
 static void log_print(const char *arg) {
@@ -214,35 +190,39 @@ int counter_wait(svr_counter_t *cst, int limit )
 
 /*++++++++++++++++++++++++++++++++++++++
 
-  Loading the radix tree. Started as a separate thread.
+  Initialize the radix tree - all trees are added to the forest in
+  a locked state.  As each tree finishes loading (from radix_load()),
+  it is unlocked.
 
-  Author:
-    marek
   ++++++++++++++++++++++++++++++++++++++*/
-void radix_init(void){
-  int i;
-  ca_dbSource_t *source_hdl;
+static void 
+radix_init(void)
+{
+    int i;
+    ca_dbSource_t *source_hdl;
 
-  wr_log_set(0);
-  /* this needs to be done in two loops, 
-     because the trees must be created asap (first loop)
-     and then locked until they are populated in the second loop
-  */
- 
-  TH_init_read_write_lockw(&rx_forest_rwlock);
-    
-  for(i=0; (source_hdl = ca_get_SourceHandleByPosition(i))!=NULL ; i++){   
-    dieif( RP_init_trees( source_hdl ) != RP_OK );
-  }
-  
-  for(i=0; (source_hdl = ca_get_SourceHandleByPosition(i))!=NULL ; i++){   
-    dieif( RP_sql_load_reg( source_hdl ) != RP_OK ); 
-  }
-  
-  wr_log_set(0); /* switch on/off the memory leak detector */
-/*  pthread_mutex_unlock( &radix_initializing_lock );  */
+    TH_init_read_write_lockw(&rx_forest_rwlock);
+    for(i=0; (source_hdl = ca_get_SourceHandleByPosition(i))!=NULL ; i++){   
+        dieif( RP_init_trees( source_hdl ) != RP_OK );
+    }
+}
 
-  pthread_exit((void *)0);
+/*++++++++++++++++++++++++++++++++++++++
+
+  Load the radix tree - all trees are initialized from the appropriate
+  SQL tables.  As each tree finishes loading (from radix_load()), it is
+  unlocked.  They must be created by the radix_init() function.
+
+  ++++++++++++++++++++++++++++++++++++++*/
+static void
+radix_load(void)
+{
+    int i;
+    ca_dbSource_t *source_hdl;
+
+    for(i=0; (source_hdl = ca_get_SourceHandleByPosition(i))!=NULL ; i++){   
+        dieif( RP_sql_load_reg( source_hdl ) != RP_OK ); 
+    }
 }
 
 
@@ -253,8 +233,8 @@ void radix_init(void){
 * but at most <delay> seconds                               *
 *                                                           *
 * Returns:                                                  *
-* 1 - timeout                                               *
-* 0 - shutdown                                              *
+* 0 - timeout                                               *
+* 1 - shutdown                                              *
 *                                                           *
 ************************************************************/
 
@@ -268,7 +248,7 @@ int SV_sleep(int delay)
   sleep(TIME_SLICE);
   elapsed_time+=TIME_SLICE;
  }
- if(elapsed_time<delay)return(1); else return(0);	
+ if(do_server)return(0); else return(1);	
 }
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -390,7 +370,7 @@ void *SV_do_child(void *varg)
   ER_dbg_va(FAC_TH, ASP_TH_NEW, 
 	    "%s threads-- = %d", args->name, curclients); 
 
-  free(args);
+  UT_free(args);
 
   return NULL; /* exit the thread */
 } /* SV_do_child */
@@ -423,16 +403,32 @@ static void  *main_loop(void *arg) {
   char loopname[32];
   int children;
   char chnum[16];
+  struct pollfd pfd[2];
 
   snprintf(loopname, 32, "s-%s", argset->name);
 
   TA_add(0, loopname);
+
+  pfd[0].fd = argset->accept_sock;
+  pfd[0].events = POLLIN;
+  pfd[1].fd = SV_shutdown_recv_fd;
+  pfd[1].events = POLLIN;
   
   while( CO_get_do_server() != 0 ) {
     /* check the number of clients, do not proceed until it's below limit */
     children = counter_wait( argset->counter, argset->limit );
     snprintf(chnum, 16, "%d", children);
     TA_setactivity(chnum); /* display the current number of children */
+
+    /* check for input */
+    if (poll(pfd, 2, -1) == -1) {
+        ER_perror(FAC_SV, 1, "%s got error %d on poll; %s", argset->name,
+                  errno, strerror(errno));
+        continue;
+    }
+
+    /* see if we're shutting down */
+    if (pfd[1].revents != 0) break;
     
     /* wait for new connections */
     argset->conn_sock = SK_accept_connection(argset->accept_sock);
@@ -462,7 +458,7 @@ static void  *main_loop(void *arg) {
        and expect SV_do_child to free it after use. 
        Caveat: the counter remains where it was, we just copy the pointer.
     */
-    argcopy = malloc( sizeof(svr_args) );
+    argcopy = UT_malloc( sizeof(svr_args) );
     memcpy( argcopy, argset, sizeof(svr_args) );
     TH_create( SV_do_child, (void *)argcopy );
   } 
@@ -470,8 +466,7 @@ static void  *main_loop(void *arg) {
   TA_delete();
   ER_dbg_va(FAC_TH, ASP_TH_NEW, "Exiting from the main loop");
 
-  pthread_exit((void *)0);
-  return NULL; /* stupid compilers. */
+  return NULL;
 } /* main_loop() */
 
 /* SV_concurrent_server() */
@@ -499,14 +494,14 @@ void SV_concurrent_server(int sock, int limit,  char *name,
 {
   svr_args *args;
   
-  dieif( wr_calloc((void **)&args, 1, sizeof(svr_args)) != UT_OK);  
+  args = (svr_args *)UT_calloc(1, sizeof(svr_args));
 
   args->accept_sock=sock;
   args->limit=limit;
   args->name=name;
   args->function=do_function;
+  args->counter = (svr_counter_t *)UT_calloc(1, sizeof(svr_counter_t));
 
-  dieif( wr_calloc((void **)&(args->counter),1,sizeof(svr_counter_t)) != UT_OK);  
   pthread_mutex_init( &(args->counter->lock), NULL );
   pthread_cond_init(  &(args->counter->cond), NULL );
   args->counter->count = 0;
@@ -537,21 +532,24 @@ void SV_concurrent_server(int sock, int limit,  char *name,
   +html+ <A HREF=".DBrc">.properties</A>
 
   ++++++++++++++++++++++++++++++++++++++*/
-void SV_start() {
+int SV_start(char *pidfile) {
   int whois_port = -1;
   int config_port = -1;
   int mirror_port = -1; 
   int update_port = -1;
   int update_mode = 0;
-  int fdes[2];
+  int pid_fd;
+  int nwrite;
   struct timeval tval;
   char starttime[128];
+  char server_pid[16];
   ca_dbSource_t *source_hdl;
   char *source_name;
   int source;
   char *db_host, *db_name, *db_user, *db_passwd;
   int db_port;
   SQ_connection_t *db_connection;
+  int shutdown_pipe[2];
 
   /* Store the starting time */
   gettimeofday(&tval, NULL);
@@ -561,24 +559,35 @@ void SV_start() {
   ctime_r(&SV_starttime, starttime);
   ER_inf_va(FAC_SV, 0xFFFFFF, "Server is started %s", starttime);
 
-  
-  
-  
-  /* Create interrupt pipe */
-  /* Writing to this pipe will cause sleeping threads */
-  /* to wake up */
-  fprintf(stderr, "Creating an interrupt pipe\n");
-  if(pipe(fdes)==-1) {
-   printf("Cannot open interrupt pipe\n");
-   exit(-1);
-  } 
-  /* Save the pipe descriptors in sv_lock array */
-  sv_lockfd[WLOCK_SHTDOWN]=fdes[0];
-  sv_lockfd[LOCK_SHTDOWN]=fdes[1];
+  /* Create our shutdown pipe */
+  if (pipe(shutdown_pipe) != 0) {
+      ER_perror(FAC_SV, 1, 
+                "error %d creating shutdown pipe; %s", errno, strerror(errno));
+      return(-1); 
+  }
+  SV_shutdown_send_fd = shutdown_pipe[1];
+  SV_shutdown_recv_fd = shutdown_pipe[0];
 
+  /* Store the PID of the process */
+  SV_pidfile = pidfile;
+  /* create the file read-writable by the process owner */
+  if((pid_fd=open(SV_pidfile,  O_CREAT | O_TRUNC | O_WRONLY, 0600))==-1) {
+      ER_perror(FAC_SV, 1, "cannot open pid file %s", SV_pidfile);     
+      return(-1);
+  }
+  sprintf(server_pid, "%d", (int)getpid()); 
+  nwrite=write(pid_fd, server_pid, strlen(server_pid) );
+  close(pid_fd);
+  if(nwrite != strlen(server_pid)) { 
+      ER_perror(FAC_SV, 1, "cannot write to pid file %s", SV_pidfile);
+      return(-1); 
+  }
+  
+  
   /* Initialise modules */
   SK_init();
-  
+  PW_init();
+   
   /* Initialise the access control list. */
   AC_build();
   AC_acc_load();
@@ -610,7 +619,7 @@ void SV_start() {
   /* config interface socket */
   SV_config_sock = SK_getsock(SOCK_STREAM, config_port, 5, INADDR_ANY);
   /* nrt socket */
-  SV_mirror_sock = SK_getsock(SOCK_STREAM,mirror_port, 5, INADDR_ANY);
+  SV_mirror_sock = SK_getsock(SOCK_STREAM,mirror_port, 128, INADDR_ANY);
   
   /* Check every Database and create sockets */
   /* we need first to create and bind all of them */
@@ -630,10 +639,10 @@ void SV_start() {
      TR_recover(db_connection);
      /* free resources */
      SQ_close_connection(db_connection);
-     free(db_host);
-     free(db_name);
-     free(db_user);
-     free(db_passwd);
+     UT_free(db_host);
+     UT_free(db_name);
+     UT_free(db_user);
+     UT_free(db_passwd);
      
      update_mode = ca_get_srcmode(source_hdl);
      if(IS_UPDATE(update_mode)) {
@@ -642,20 +651,16 @@ void SV_start() {
        printf("XXX htons(update_port)=%d\n", update_port);
        /* XXX ask AMRM to change the name of the function */
  
-       SV_update_sock[source] = SK_getsock(SOCK_STREAM, update_port, 5, INADDR_ANY);
+       SV_update_sock[source] = SK_getsock(SOCK_STREAM, update_port, 128, INADDR_ANY);
      }
      else SV_update_sock[source] = 0;
   }   
   SV_update_sock[source+1]=-1; /* end of socket array */
    
-   /* Initialise the radix tree (separate thread[s])
+  /* Initialise the radix trees
      already can allow socket connections, because the trees will 
      be created locked, and will be unlocked when loaded */
-
-/*   pthread_mutex_lock( &radix_initializing_lock );  */
-  TH_create((void *(*)(void *))radix_init, NULL);
-/*  pthread_mutex_lock( &radix_initializing_lock );  */
-  
+  radix_init();
  
   /* Now.... accept() calls block until they get a connection
      so to listen on more than one port we need more
@@ -667,6 +672,10 @@ void SV_start() {
   SV_concurrent_server(SV_config_sock, 0, "config", PC_interact);
   /* Create master thread for mirror threads */
   SV_concurrent_server(SV_mirror_sock, 0, "mirror", PM_interact);
+
+  /* Load the radix trees - this must be done before the 
+     updates are allowed to proceed */
+  radix_load();
   
   /* Walk through the sources and */
   /* run update thread for every source with CANUPD == 'y' */
@@ -691,11 +700,12 @@ void SV_start() {
 	     fprintf(stderr,"Source [%s] Mode STATIC\n", source_name);
 	     ER_inf_va(FAC_SV, 0xFFFFFF, "Source [%s] Mode STATIC", source_name);
      }
-     free(source_name); /* because ca_* functions return copies */   
+     UT_free(source_name); /* because ca_* functions return copies */   
    }    
-
+  /* terminate the thread */
+  /* XXX not return becase then we terminate the whole process */
   pthread_exit(NULL);
-
+  return(1); /* we will never reach this point */
 } /* SV_start() */
 
 /* SV_shutdown() */
@@ -732,12 +742,16 @@ char shuttime[100];
  fprintf(stderr, print_buf);
  /*log_print(print_buf); */
  strcpy(print_buf, "");
-
+ 
+ /* wait for all updates to complete */
+ /* XXX may be changed with blocking interface for stop updates */
+ sleep(5);
+ 
  /* Store the shutdown time */
   gettimeofday(&tval, NULL);
   shutdowntime = tval.tv_sec;/* seconds since Jan. 1, 1970 */
 
-  /* Log the starting time */
+  /* Log the sshutdown time */
   ctime_r(&shutdowntime, shuttime);
   ER_inf_va(FAC_SV, 0xFFFFFF, "Server shutdown %s", shuttime);
 
@@ -745,14 +759,27 @@ char shuttime[100];
     
  /* Wake up all sleeping threads */
  fprintf(stderr, "Going to wake sleeping threads up\n");
- write(sv_lockfd[WLOCK_SHTDOWN], " ", 1); 
 
+ /* Delete the pid file to indicate normal shutdown */
+ if(unlink(SV_pidfile)==-1) {
+      ER_perror(FAC_SV, 1, "cannot delete pid file %s:", SV_pidfile);     
+ }
+
+ /* write to shutdown pipe */
+ if (write(SV_shutdown_send_fd, "", 1) != 1) {
+      ER_perror(FAC_SV, 1, "error %d writing to shutdown pipe; %s", 
+                errno, strerror(errno));
+ }
+
+ 
+/* don't do this, it's not polite  ;) */
+#if 0
  /* CLose all listening sockets, so accept call exits */
  close(SV_whois_sock);
  close(SV_config_sock);
  close(SV_mirror_sock);
  for (source=0; SV_update_sock[source]!=-1; source++)
 	 if(SV_update_sock[source]!=0)close(SV_update_sock[source]);
- 
+#endif /* 0 */ 
  
 } /* SV_shutdown() */
