@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2001                      RIPE NCC
+Copyright (c) 2001,2002                 RIPE NCC
 
 
 All Rights Reserved
@@ -20,7 +20,7 @@ AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
-/* $Id: whois3.c,v 1.3 2001/09/17 14:29:30 shane Exp $ */
+/* $Id: whois3.c,v 1.6 2002/02/19 17:22:59 shane Exp $ */
 
 #include <string.h>
 #include <stdlib.h>
@@ -35,6 +35,8 @@ OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include <unistd.h>
 
 
+/* help text is defined in this file as flag_txt[] */
+#include "flags.h"
 
 /*
  * CONSTANTS 
@@ -61,13 +63,14 @@ OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
   the result in the "in" descriptor.  The "out" buffer may have either no
   buffering or line buffering, but must NOT have full buffering.
 
-  The routine then outputs each line the server returns, until either
-  two consecutive blank lines appear in the server response, or the
-  server ends the connection.
+  The routine then outputs each line the server returns, until the server
+  ends the connection.  If the "check_for_blank" variable is set to
+  non-zero, then the routine will also return when two consecutive blank
+  lines appear in the server response.
  */
 
 int 
-whois_query(FILE *in, FILE *out, char *query)
+whois_query(FILE *in, FILE *out, char *query, int check_for_blank)
 {
     char buf[1024];
     int last_line_blank;
@@ -103,7 +106,7 @@ whois_query(FILE *in, FILE *out, char *query)
             /* if line is empty */
             if (!strcmp(buf, "\n")) {
                 /* if the last line was also blank, we're done */
-                if (last_line_blank) {
+                if (check_for_blank && last_line_blank) {
                     return 1;
                 }
                 last_line_blank = 1;
@@ -136,6 +139,13 @@ usage_error(const char *exename)
     fprintf(stderr, 
             "%s: [-h host | --host=host] [-p port | --port=port] -k | query\n",
             exename);
+    fprintf(stderr, "\n");
+    fprintf(stderr, 
+            "When querying a Whois server running RIPE software, you may\n");
+    fprintf(stderr, 
+            "use the following flags:\n");
+    fprintf(stderr, "\n");
+    fputs(flag_txt, stderr);
     exit(1);
 }
 
@@ -310,6 +320,11 @@ main(int argc, char *argv[])
 	    p++;
         }
 
+        /* check for help, so we can give local usage */
+        else if (!strcmp(argv[p], "-?")) {
+            usage_error(exename);
+        }
+
         /* other flags or arguments */
         else {
             /* check to see if -k was used - this will cause an error below,
@@ -449,7 +464,7 @@ main(int argc, char *argv[])
         }
 
         /* now send our query to the server */
-        whois_query(whois_in, whois_out, query);
+        whois_query(whois_in, whois_out, query, 0);
     }
 
     /* otherwise we're in "batch" mode - read each query a line at a time */
@@ -472,7 +487,7 @@ main(int argc, char *argv[])
                 fprintf(stderr, "%s: query line too long\n", exename);
                 exit(1);
             }
-            whois_query(whois_in, whois_out, query);
+            whois_query(whois_in, whois_out, query, 1);
         }
 
         /* exit persistent mode */
